@ECHO OFF
REM #############################  Decrypt.bat  ################################
REM # You can use this batch file to decrypt backup files using command-line.  #
REM # Just customize the "User Define Section" below with values for your      #
REM # decrypt action.                                                          #
REM ############################################################################

REM ####################  Start: User Defined Section  #########################

REM ---------------------------------  ID_DIR  ---------------------------------
REM | Path to the backup set ID directory                                      |
REM | e.g. SET ID_DIR="C:\john\files\1119083740107"                            |
REM |      where the directory 1119083740107 is under the "files" directory    |
REM |      under the [User Home] directory                                     |
REM ----------------------------------------------------------------------------
SET ID_DIR=""

REM -------------------------------  DECRYPT_TO  -------------------------------
REM | Directory to where you want files to be decrypted                        |
REM | Use "\\\\" instead of "\\" when you specify a UNC path                   |
REM | set to "" to decrypt files to original location                          |
REM | e.g. SET DECRYPT_TO="C:\Temp"                                            |
REM ----------------------------------------------------------------------------
SET DECRYPT_TO=""

REM ------------------------------  DECRYPT_FROM  ------------------------------
REM | File/Directory on the backup server that you would like to decrypt       |
REM | e.g. SET DECRYPT_FROM="C:\Data"                                          |
REM ----------------------------------------------------------------------------
SET DECRYPT_FROM=""

REM -----------------------------  POINT_IN_TIME  ------------------------------
REM | The point-in-time snapshot that you want to decrypt.                     |
REM | Use "Current" for the latest backup snapshot                             |
REM | e.g. SET POINT_IN_TIME="2006-10-04-12-57-13"                             |
REM |  or  SET POINT_IN_TIME="Current"                                         |
REM ----------------------------------------------------------------------------
SET POINT_IN_TIME="Current"

REM ------------------------------  DECRYPT_KEY  -------------------------------
REM | Your encrypting key. It will be used to decrypt your backup files        |
REM | e.g. SET DECRYPT_KEY="abc"                                               |
REM ----------------------------------------------------------------------------
SET DECRYPT_KEY=""

REM ---------------------------  RESTORE_PERMISSION  ---------------------------
REM | set to "Y" if you want to restore file permissions                       |
REM | set to "N" if you do NOT want to restore file permissions                |
REM ----------------------------------------------------------------------------
SET RESTORE_PERMISSION="N"

REM ----------------------------  SKIP_INVALID_KEY  ----------------------------
REM | set to "Y" if you want to skip decrypt file with invalid key             |
REM | set to "N" if you want to prompt user to input a correct key             |
REM ----------------------------------------------------------------------------
SET SKIP_INVALID_KEY="N"

REM ------------------------------  SYNC_OPTION  -------------------------------
REM | Delete extra files                                                       |
REM | set to "Y" if you want to enable sync option                             |
REM | set to "N" if you do NOT want to enable sync option                      |
REM | set to "" to prompt for selection                                        |
REM ----------------------------------------------------------------------------
SET SYNC_OPTION="N"

REM -------------------------  REPLACE_EXISTING_FILE  --------------------------
REM | set to "-all" to replace all existing file(s) of the same filename       |
REM | set to "-none" to skip all existing file(s) with the same filename       |
REM | set to "" to prompt for selection                                        |
REM ----------------------------------------------------------------------------
SET REPLACE_EXISTING_FILE="-all"

REM ---------------------------------  FILTER  ---------------------------------
REM | Filter out what files you want to decrypt                                |
REM | -Pattern=xxx-Type=yyy-Target=zzz                                         |
REM | where xxx is the filter pattern,                                         |
REM |       yyy is the filter type, whice can be one of the following:         |
REM |           [exact | exactMatchCase | contains | containsMatchCase|        |
REM |            startWith | startWithMatchCase | endWith | endWithMatchCase]  |
REM |       zzz is the filter target, which can be one of the following:       |
REM |           [toFile | toFileDir | toDir]                                   |
REM |                                                                          |
REM | e.g. SET FILTER="-Pattern=.txt-Type=exact-Target=toFile"                 |
REM ----------------------------------------------------------------------------
SET FILTER=""

REM ####################  END: User Defined Section  ###########################

SET APP_HOME=..
SET JAVA_HOME=%APP_HOME%\jvm
SET JAVA_EXE=%JAVA_HOME%\bin\java.exe
SET JAVA_OPTS=-Xms128m -Xmx768m -Dsun.java2d.noddraw 
SET JAVA_LIB_PATH=-Djava.library.path=%APP_HOME%\bin
SET PATH=%JAVA_HOME%\bin;%PATH%
SET CLASSPATH=%APP_HOME%\bin;%APP_HOME%\bin\obc.jar;%APP_HOME%\bin\obc-lib.jar

REM #############################################################################
ECHO - 
ECHO APP_HOME=%APP_HOME%
ECHO JAVA_HOME=%JAVA_HOME%
ECHO JAVA_EXE=%JAVA_EXE%
ECHO JAVA_OPTS=%JAVA_OPTS%
ECHO JAVA_LIB_PATH=%JAVA_LIB_PATH%
ECHO PATH=%PATH%
ECHO CLASSPATH=%CLASSPATH%
ECHO - 

ECHO Decrypting Backup Set ID Directory - '%ID_DIR%' ...
@ECHO ON
%JAVA_EXE% %JAVA_LIB_PATH% -cp %CLASSPATH% %JAVA_OPTS% Decrypt -to=%DECRYPT_TO% -from=%DECRYPT_FROM% -source=%ID_DIR% %REPLACE_EXISTING_FILE% -date=%POINT_IN_TIME% -key=%DECRYPT_KEY% -setPermission=%RESTORE_PERMISSION% -skipInvalidKey=%SKIP_INVALID_KEY% -sync=%SYNC_OPTION% -Filter=%FILTER% -AppHome=%APP_HOME%
PAUSE